@@ -1,5 +1,6 @@
 # --
-# Copyright (C) 2001-2016 OTRS AG, http://otrs.com/
+# Kernel/Modules/AgentTicketResponsibleView.pm - to view all locked tickets
+# Copyright (C) 2001-2015 OTRS AG, http://otrs.com/
 # --
 # This software comes with ABSOLUTELY NO WARRANTY. For details, see
 # the enclosed file COPYING for license information (AGPL). If you
@@ -154,8 +155,6 @@ sub Run {
             Refresh => $Refresh,
         );
         $Output .= $Self->{LayoutObject}->NavigationBar();
-        $Self->{LayoutObject}->Print( Output => \$Output );
-        $Output = '';
     }
 
     # get locked  viewable tickets...
@@ -278,8 +277,9 @@ sub Run {
         }
 
         my @OriginalViewableTicketsTmp;
+        TICKETID:
         for my $TicketIDAll (@OriginalViewableTicketsAll) {
-            next if $OriginalViewableTicketsNotNew{$TicketIDAll};
+            next TICKETID if $OriginalViewableTicketsNotNew{$TicketIDAll};
             push @OriginalViewableTicketsTmp, $TicketIDAll;
         }
         @OriginalViewableTickets = @OriginalViewableTicketsTmp;
@@ -297,8 +297,9 @@ sub Run {
         }
 
         my @ViewableTicketsTmp;
+        TICKETID:
         for my $TicketIDAll (@ViewableTicketsAll) {
-            next if $ViewableTicketsNotNew{$TicketIDAll};
+            next TICKETID if $ViewableTicketsNotNew{$TicketIDAll};
             push @ViewableTicketsTmp, $TicketIDAll;
         }
         @ViewableTickets = @ViewableTicketsTmp;
@@ -434,6 +435,9 @@ sub Run {
         ColumnFilterForm    => {
             Filter => $Self->{Filter} || '',
         },
+
+        # do not print the result earlier, but return complete content
+        Output => 1,
     );
 
     $Output .= $Self->{LayoutObject}->Footer();
