@@ -1,5 +1,6 @@
 # --
-# Copyright (C) 2001-2016 OTRS AG, http://otrs.com/
+# Kernel/System/Web/InterfaceAgent.pm - the agent interface file (incl. auth)
+# Copyright (C) 2001-2015 OTRS AG, http://otrs.com/
 # --
 # This software comes with ABSOLUTELY NO WARRANTY. For details, see
 # the enclosed file COPYING for license information (AGPL). If you
@@ -11,18 +12,23 @@ package Kernel::System::Web::InterfaceAgent;
 use strict;
 use warnings;
 
-use Kernel::Config;
-use Kernel::System::Log;
-use Kernel::System::Main;
-use Kernel::System::Encode;
-use Kernel::System::Time;
-use Kernel::System::Web::Request;
-use Kernel::System::DB;
-use Kernel::System::Auth;
-use Kernel::System::AuthSession;
-use Kernel::System::User;
-use Kernel::System::Group;
-use Kernel::Output::HTML::Layout;
+our @ObjectDependencies = (
+    'Kernel::Config',
+    'Kernel::Language',
+    'Kernel::Output::HTML::Layout',
+    'Kernel::System::Auth',
+    'Kernel::System::AuthSession',
+    'Kernel::System::DB',
+    'Kernel::System::Encode',
+    'Kernel::System::Email',
+    'Kernel::System::Group',
+    'Kernel::System::Log',
+    'Kernel::System::Main',
+    'Kernel::System::Ticket',
+    'Kernel::System::Time',
+    'Kernel::System::User',
+    'Kernel::System::Web::Request',
+);
 
 =head1 NAME
 
@@ -40,45 +46,56 @@ the global agent web interface (incl. auth, session, ...)
 
 =item new()
 
-create agent web interface object
+create agent web interface object. Do not use it directly, instead use:
 
-    use Kernel::System::Web::InterfaceAgent;
-
-    my $Debug = 0;
-    my $InterfaceAgent = Kernel::System::Web::InterfaceAgent->new(
-        Debug      => $Debug,
-        WebRequest => CGI::Fast->new(), # optional, e. g. if fast cgi is used, the CGI object is already provided
+    use Kernel::System::ObjectManager;
+    my $Debug = 0,
+    local $Kernel::OM = Kernel::System::ObjectManager->new(
+        'Kernel::System::Web::InterfaceAgent' => {
+            Debug   => 0,
+            WebRequest => CGI::Fast->new(), # optional, e. g. if fast cgi is used,
+                                            # the CGI object is already provided
+        }
     );
+    my $InterfaceAgent = $Kernel::OM->Get('Kernel::System::Web::InterfaceAgent');
 
 =cut
 
 sub new {
     my ( $Type, %Param ) = @_;
-
-    # allocate new hash for object
     my $Self = {};
     bless( $Self, $Type );
 
+    # Performance log
+    $Self->{PerformanceLogStart} = time();
+
     # get debug level
     $Self->{Debug} = $Param{Debug} || 0;
 
-    # performance log
-    $Self->{PerformanceLogStart} = time();
+    $Self->{ConfigObject} = $Kernel::OM->Get('Kernel::Config');
+    $Kernel::OM->ObjectParamAdd(
+        'Kernel::System::Log' => {
+            LogPrefix => $Self->{ConfigObject}->Get('CGILogPrefix'),
+        },
+        'Kernel::System::Web::Request' => {
+            WebRequest => $Param{WebRequest} || 0,
+        },
 
-    # create common framework objects 1/2
-    $Self->{ConfigObject} = Kernel::Config->new();
-    $Self->{LogObject}    = Kernel::System::Log->new(
-        LogPrefix => $Self->{ConfigObject}->Get('CGILogPrefix'),
-        %{$Self},
-    );
-    $Self->{EncodeObject} = Kernel::System::Encode->new( %{$Self} );
-    $Self->{MainObject}   = Kernel::System::Main->new( %{$Self} );
-    $Self->{TimeObject}   = Kernel::System::Time->new( %{$Self} );
-    $Self->{ParamObject}  = Kernel::System::Web::Request->new(
-        %{$Self},
-        WebRequest => $Param{WebRequest} || 0,
+        # Don't autoconnect as this would cause internal server errors on failure.
+        'Kernel::System::DB' => {
+            AutoConnectNo => 1,
+        },
     );
 
+    $Self->{EncodeObject}  = $Kernel::OM->Get('Kernel::System::Encode');
+    $Self->{GroupObject}   = $Kernel::OM->Get('Kernel::System::Group');
+    $Self->{LogObject}     = $Kernel::OM->Get('Kernel::System::Log');
+    $Self->{MainObject}    = $Kernel::OM->Get('Kernel::System::Main');
+    $Self->{ParamObject}   = $Kernel::OM->Get('Kernel::System::Web::Request');
+    $Self->{SessionObject} = $Kernel::OM->Get('Kernel::System::AuthSession');
+    $Self->{TimeObject}    = $Kernel::OM->Get('Kernel::System::Time');
+    $Self->{UserObject}    = $Kernel::OM->Get('Kernel::System::User');
+
     # debug info
     if ( $Self->{Debug} ) {
         $Self->{LogObject}->Log(
@@ -125,12 +142,13 @@ sub Run {
     }
 
     # validate language
-    if ( $Param{Lang} && $Param{Lang} !~ m{\A[a-z]{2}(?:_[A-Z]{2})?\z}xms ) {
+    if ($Param{Lang} && $Param{Lang} !~ m{\A[a-z]{2}(?:_[A-Z]{2})?\z}xms) {
         delete $Param{Lang};
     }
 
     # check if the browser sends the SessionID cookie and set the SessionID-cookie
     # as SessionID! GET or POST SessionID have the lowest priority.
+    my $BrowserHasCookie = 0;
     if ( $Self->{ConfigObject}->Get('SessionUseCookie') ) {
         $Param{SessionIDCookie} = $Self->{ParamObject}->GetCookie( Key => $Param{SessionName} );
         if ( $Param{SessionIDCookie} ) {
@@ -138,6 +156,16 @@ sub Run {
         }
     }
 
+    $Kernel::OM->ObjectParamAdd(
+        'Kernel::Output::HTML::Layout' => {
+            Lang         => $Param{Lang},
+            UserLanguage => $Param{Lang},
+        },
+        'Kernel::Lanugage' => {
+            UserLanguage => $Param{Lang}
+        },
+    );
+
     my $CookieSecureAttribute;
     if ( $Self->{ConfigObject}->Get('HttpType') eq 'https' ) {
 
@@ -145,14 +173,18 @@ sub Run {
         $CookieSecureAttribute = 1;
     }
 
-    # check common objects
-    $Self->{DBObject} = Kernel::System::DB->new( %{$Self} );
-    if ( !$Self->{DBObject} || $Self->{ParamObject}->Error() ) {
-        my $LayoutObject = Kernel::Output::HTML::Layout->new(
-            %{$Self},
-            Lang => $Param{Lang},
-        );
-        if ( !$Self->{DBObject} ) {
+    $Self->{DBObject} = $Kernel::OM->Get('Kernel::System::DB');
+    my $DBCanConnect = $Self->{DBObject}->Connect();
+
+    # Restore original behaviour of Kernel::System::DB for all objects created in future
+    $Kernel::OM->ObjectParamAdd(
+        'Kernel::System::DB' => {
+            AutoConnectNo => undef,
+        },
+    );
+    if ( !$DBCanConnect || $Self->{ParamObject}->Error() ) {
+        my $LayoutObject = $Kernel::OM->Get('Kernel::Output::HTML::Layout');
+        if ( !$DBCanConnect ) {
             $LayoutObject->FatalError(
                 Comment => 'Please contact your administrator',
             );
@@ -167,27 +199,12 @@ sub Run {
         }
     }
 
-    # create common framework objects 2/2
-    $Self->{UserObject}    = Kernel::System::User->new( %{$Self} );
-    $Self->{GroupObject}   = Kernel::System::Group->new( %{$Self} );
-    $Self->{SessionObject} = Kernel::System::AuthSession->new( %{$Self} );
-
     # application and add-on application common objects
     my %CommonObject = %{ $Self->{ConfigObject}->Get('Frontend::CommonObject') };
-    for my $Key ( sort keys %CommonObject ) {
-        if ( $Self->{MainObject}->Require( $CommonObject{$Key} ) ) {
-            $Self->{$Key} = $CommonObject{$Key}->new( %{$Self} );
-        }
-        else {
+    $Self->{TicketObject} = $Kernel::OM->Get('Kernel::System::Ticket');
 
-            # print error
-            my $LayoutObject = Kernel::Output::HTML::Layout->new(
-                %{$Self},
-                Lang => $Param{Lang},
-            );
-            $LayoutObject->FatalError( Comment => 'Please contact your administrator' );
-            return;
-        }
+    for my $Key ( sort keys %CommonObject ) {
+        $Self->{$Key} //= $Kernel::OM->Get( $CommonObject{$Key} );
     }
 
     # get common application and add-on application params
@@ -200,14 +217,23 @@ sub Run {
     $Param{Action} =~ s/\W//g;
 
     # check request type
-    if ( $Param{Action} eq 'Login' ) {
+    if ( $Param{Action} eq 'PreLogin' ) {
+        my $LayoutObject = $Kernel::OM->Get('Kernel::Output::HTML::Layout');
+        $Param{RequestedURL} = $Param{RequestedURL} || "Action=AgentDashboard";
 
-        # new layout object
-        my $LayoutObject = Kernel::Output::HTML::Layout->new(
-            %{$Self},
-            Lang => $Param{Lang},
+        # login screen
+        $LayoutObject->Print(
+            Output => \$LayoutObject->Login(
+                Title => 'Login',
+                Mode  => 'PreLogin',
+                %Param,
+            ),
         );
 
+        return;
+    }
+    elsif ( $Param{Action} eq 'Login' ) {
+
         # get params
         my $PostUser = $Self->{ParamObject}->GetParam( Param => 'User' ) || '';
         my $PostPw = $Self->{ParamObject}->GetParam(
@@ -216,7 +242,7 @@ sub Run {
         ) || '';
 
         # create AuthObject
-        my $AuthObject = Kernel::System::Auth->new( %{$Self} );
+        my $AuthObject = $Kernel::OM->Get('Kernel::System::Auth');
 
         # check submitted data
         my $User = $AuthObject->Auth(
@@ -227,6 +253,27 @@ sub Run {
         # login is invalid
         if ( !$User ) {
 
+            my $Expires = '+' . $Self->{ConfigObject}->Get('SessionMaxTime') . 's';
+            if ( !$Self->{ConfigObject}->Get('SessionUseCookieAfterBrowserClose') ) {
+                $Expires = '';
+            }
+
+            $Kernel::OM->ObjectParamAdd(
+                'Kernel::Output::HTML::Layout' => {
+                    SetCookies => {
+                        OTRSBrowserHasCookie => $Self->{ParamObject}->SetCookie(
+                            Key      => 'OTRSBrowserHasCookie',
+                            Value    => 1,
+                            Expires  => $Expires,
+                            Path     => $Self->{ConfigObject}->Get('ScriptAlias'),
+                            Secure   => $CookieSecureAttribute,
+                            HTTPOnly => 1,
+                        ),
+                    },
+                    }
+            );
+            my $LayoutObject = $Kernel::OM->Get('Kernel::Output::HTML::Layout');
+
             # redirect to alternate login
             if ( $Self->{ConfigObject}->Get('LoginURL') ) {
                 $Param{RequestedURL} = $LayoutObject->LinkEncode( $Param{RequestedURL} );
@@ -245,7 +292,9 @@ sub Run {
                         Type => 'Info',
                         What => 'Message',
                         )
-                        || 'Login failed! Your user name or password was entered incorrectly.',
+                        || $LayoutObject->{LanguageObject}->Translate( $AuthObject->GetLastErrorMessage() )
+                        || $LayoutObject->{LanguageObject}
+                        ->Translate('Login failed! Your user name or password was entered incorrectly.'),
                     LoginFailed => 1,
                     User        => $User,
                     %Param,
@@ -260,17 +309,29 @@ sub Run {
             Valid => 1
         );
 
+        # check if the browser supports cookies
+
+        if ( $Self->{ParamObject}->GetCookie( Key => 'OTRSBrowserHasCookie' ) ) {
+            $Kernel::OM->ObjectParamAdd(
+                'Kernel::Output::HTML::Layout' => {
+                    BrowserHasCookie => 1,
+                },
+            );
+        }
+
         # check needed data
         if ( !$UserData{UserID} || !$UserData{UserLogin} ) {
 
             # redirect to alternate login
             if ( $Self->{ConfigObject}->Get('LoginURL') ) {
-                print $LayoutObject->Redirect(
+                print $Kernel::OM->Get('Kernel::Output::HTML::Layout')->Redirect(
                     ExtURL => $Self->{ConfigObject}->Get('LoginURL') . '?Reason=SystemError',
                 );
                 return;
             }
 
+            my $LayoutObject = $Kernel::OM->Get('Kernel::Output::HTML::Layout');
+
             # show need user data error message
             $LayoutObject->Print(
                 Output => \$LayoutObject->Login(
@@ -314,6 +375,7 @@ sub Run {
             my $Error = $Self->{SessionObject}->SessionIDErrorMessage() || '';
 
             # output error message
+            my $LayoutObject = $Kernel::OM->Get('Kernel::Output::HTML::Layout');
             $LayoutObject->Print(
                 Output => \$LayoutObject->Login(
                     Title   => 'Login',
@@ -328,7 +390,6 @@ sub Run {
         if (
             $Self->{ConfigObject}->Get('TimeZoneUser')
             && $Self->{ConfigObject}->Get('TimeZoneUserBrowserAutoOffset')
-            && $LayoutObject->{BrowserJavaScriptSupport}
             )
         {
             my $TimeOffset = $Self->{ParamObject}->GetParam( Param => 'TimeOffset' ) || 0;
@@ -339,6 +400,7 @@ sub Run {
                 $TimeOffset = $TimeOffset / 60;
                 $TimeOffset =~ s/-/+/;
             }
+
             $Self->{UserObject}->SetPreferences(
                 UserID => $UserData{UserID},
                 Key    => 'UserTimeZone',
@@ -357,45 +419,60 @@ sub Run {
             $Expires = '';
         }
 
-        $LayoutObject = Kernel::Output::HTML::Layout->new(
-            SetCookies => {
-                SessionIDCookie => $Self->{ParamObject}->SetCookie(
-                    Key      => $Param{SessionName},
-                    Value    => $NewSessionID,
-                    Expires  => $Expires,
-                    Path     => $Self->{ConfigObject}->Get('ScriptAlias'),
-                    Secure   => scalar $CookieSecureAttribute,
-                    HTTPOnly => 1,
-                ),
+        my $SecureAttribute;
+        if ( $Self->{ConfigObject}->Get('HttpType') eq 'https' ) {
+
+            # Restrict Cookie to HTTPS if it is used.
+            $SecureAttribute = 1;
+        }
+
+        $Kernel::OM->ObjectParamAdd(
+            'Kernel::Output::HTML::Layout' => {
+                SetCookies => {
+                    SessionIDCookie => $Self->{ParamObject}->SetCookie(
+                        Key      => $Param{SessionName},
+                        Value    => $NewSessionID,
+                        Expires  => $Expires,
+                        Path     => $Self->{ConfigObject}->Get('ScriptAlias'),
+                        Secure   => scalar $CookieSecureAttribute,
+                        HTTPOnly => 1,
+                    ),
+                    OTRSBrowserHasCookie => $Self->{ParamObject}->SetCookie(
+                        Key      => 'OTRSBrowserHasCookie',
+                        Value    => '',
+                        Expires  => '-1y',
+                        Path     => $Self->{ConfigObject}->Get('ScriptAlias'),
+                        Secure   => $CookieSecureAttribute,
+                        HTTPOnly => 1,
+                    ),
+                },
+                SessionID   => $NewSessionID,
+                SessionName => $Param{SessionName},
             },
-            SessionID   => $NewSessionID,
-            SessionName => $Param{SessionName},
-            %{$Self},
         );
 
         # redirect with new session id and old params
         # prepare old redirect URL -- do not redirect to Login or Logout (loop)!
-        if ( $Param{RequestedURL} =~ /Action=(Logout|Login|LostPassword)/ ) {
+        if ( $Param{RequestedURL} =~ /Action=(Logout|Login|LostPassword|PreLogin)/ ) {
             $Param{RequestedURL} = '';
         }
 
         # redirect with new session id
-        print $LayoutObject->Redirect( OP => $Param{RequestedURL} );
+        print $Kernel::OM->Get('Kernel::Output::HTML::Layout')->Redirect(
+            OP    => $Param{RequestedURL},
+            Login => 1,
+        );
         return 1;
     }
 
     # logout
     elsif ( $Param{Action} eq 'Logout' ) {
 
+        my $LayoutObject = $Kernel::OM->Get('Kernel::Output::HTML::Layout');
+
         # check session id
         if ( !$Self->{SessionObject}->CheckSessionID( SessionID => $Param{SessionID} ) ) {
 
-            # new layout object
-            my $LayoutObject = Kernel::Output::HTML::Layout->new(
-                %{$Self},
-                Lang => $Param{Lang},
-            );
-
             # redirect to alternate login
             if ( $Self->{ConfigObject}->Get('LoginURL') ) {
                 $Param{RequestedURL} = $LayoutObject->LinkEncode( $Param{RequestedURL} );
@@ -410,7 +487,7 @@ sub Run {
             $LayoutObject->Print(
                 Output => \$LayoutObject->Login(
                     Title   => 'Logout',
-                    Message => 'Session invalid. Please log in again.',
+                    Message => $LayoutObject->{LanguageObject}->Translate('Session invalid. Please log in again.'),
                     %Param,
                 ),
             );
@@ -422,22 +499,24 @@ sub Run {
             SessionID => $Param{SessionID},
         );
 
-        # create new LayoutObject with new '%Param' and '%UserData'
-        my $LayoutObject = Kernel::Output::HTML::Layout->new(
-            SetCookies => {
-                SessionIDCookie => $Self->{ParamObject}->SetCookie(
-                    Key      => $Param{SessionName},
-                    Value    => '',
-                    Expires  => '-1y',
-                    Path     => $Self->{ConfigObject}->Get('ScriptAlias'),
-                    Secure   => scalar $CookieSecureAttribute,
-                    HTTPOnly => 1,
-                ),
+        # create a new LayoutObject with %UserData
+        $Kernel::OM->ObjectParamAdd(
+            'Kernel::Output::HTML::Layout' => {
+                SetCookies => {
+                    SessionIDCookie => $Self->{ParamObject}->SetCookie(
+                        Key      => $Param{SessionName},
+                        Value    => '',
+                        Expires  => '-1y',
+                        Path     => $Self->{ConfigObject}->Get('ScriptAlias'),
+                        Secure   => scalar $CookieSecureAttribute,
+                        HTTPOnly => 1,
+                    ),
+                },
+                %UserData,
             },
-            %{$Self},
-            %Param,
-            %UserData,
         );
+        $Kernel::OM->ObjectsDiscard( Objects => ['Kernel::Output::HTML::Layout'] );
+        $LayoutObject = $Kernel::OM->Get('Kernel::Output::HTML::Layout');
 
         # Prevent CSRF attacks
         $LayoutObject->ChallengeTokenCheck();
@@ -460,10 +539,16 @@ sub Run {
         }
 
         # show logout screen
+        my $LogoutMessage = $LayoutObject->{LanguageObject}->Translate(
+            'Logout successful. Thank you for using %s!',
+            $Self->{ConfigObject}->Get("ProductName"),
+        );
+
         $LayoutObject->Print(
             Output => \$LayoutObject->Login(
-                Title   => 'Logout',
-                Message => 'Logout successful. Thank you for using %s!", "$Config{"ProductName"}',
+                Title       => 'Logout',
+                Message     => $LogoutMessage,
+                MessageType => 'Logout',
                 %Param,
             ),
         );
@@ -473,11 +558,7 @@ sub Run {
     # user lost password
     elsif ( $Param{Action} eq 'LostPassword' ) {
 
-        # new layout object
-        my $LayoutObject = Kernel::Output::HTML::Layout->new(
-            %{$Self},
-            Lang => $Param{Lang},
-        );
+        my $LayoutObject = $Kernel::OM->Get('Kernel::Output::HTML::Layout');
 
         # check feature
         if ( !$Self->{ConfigObject}->Get('LostPassword') ) {
@@ -486,7 +567,7 @@ sub Run {
             $LayoutObject->Print(
                 Output => \$LayoutObject->Login(
                     Title   => 'Login',
-                    Message => 'Feature not active!',
+                    Message => $LayoutObject->{LanguageObject}->Translate('Feature not active!'),
                 ),
             );
             return;
@@ -502,6 +583,7 @@ sub Run {
                 Key   => 'UserToken',
                 Value => $Token,
             );
+            USERS:
             for my $UserID ( sort keys %UserList ) {
                 my %UserData = $Self->{UserObject}->GetUserData(
                     UserID => $UserID,
@@ -509,7 +591,7 @@ sub Run {
                 );
                 if (%UserData) {
                     $User = $UserData{UserLogin};
-                    last;
+                    last USERS;
                 }
             }
         }
@@ -535,7 +617,7 @@ sub Run {
         }
 
         # create email object
-        my $EmailObject = Kernel::System::Email->new( %{$Self} );
+        my $EmailObject = $Kernel::OM->Get('Kernel::System::Email');
 
         # send password reset token
         if ( !$Token ) {
@@ -624,12 +706,15 @@ sub Run {
             );
             return;
         }
+        my $Message = $LayoutObject->{LanguageObject}->Translate(
+            'Sent new password to %s. Please check your email.',
+            $UserData{UserEmail},
+        );
         $LayoutObject->Print(
             Output => \$LayoutObject->Login(
-                Title => 'Login',
-                Message =>
-                    "Sent new password to \%s. Please check your email.\", \"$UserData{UserEmail}",
-                User => $User,
+                Title   => 'Login',
+                Message => $Message,
+                User    => $User,
                 %Param,
             ),
         );
@@ -639,20 +724,15 @@ sub Run {
     # show login site
     elsif ( !$Param{SessionID} ) {
 
-        # new layout object
-        my $LayoutObject = Kernel::Output::HTML::Layout->new(
-            %{$Self},
-            Lang => $Param{Lang},
-        );
-
         # create AuthObject
-        my $AuthObject = Kernel::System::Auth->new( %{$Self} );
+        my $AuthObject   = $Kernel::OM->Get('Kernel::System::Auth');
+        my $LayoutObject = $Kernel::OM->Get('Kernel::Output::HTML::Layout');
         if ( $AuthObject->GetOption( What => 'PreAuth' ) ) {
 
             # automatic login
             $Param{RequestedURL} = $LayoutObject->LinkEncode( $Param{RequestedURL} );
             print $LayoutObject->Redirect(
-                OP => "Action=Login&RequestedURL=$Param{RequestedURL}",
+                OP => "Action=PreLogin&RequestedURL=$Param{RequestedURL}",
             );
             return;
         }
@@ -683,30 +763,34 @@ sub Run {
         # check session id
         if ( !$Self->{SessionObject}->CheckSessionID( SessionID => $Param{SessionID} ) ) {
 
-            # create new LayoutObject with new '%Param'
-            my $LayoutObject = Kernel::Output::HTML::Layout->new(
-                SetCookies => {
-                    SessionIDCookie => $Self->{ParamObject}->SetCookie(
-                        Key      => $Param{SessionName},
-                        Value    => '',
-                        Expires  => '-1y',
-                        Path     => $Self->{ConfigObject}->Get('ScriptAlias'),
-                        Secure   => scalar $CookieSecureAttribute,
-                        HTTPOnly => 1,
-                    ),
+            # put '%Param' into LayoutObject
+            $Kernel::OM->ObjectParamAdd(
+                'Kernel::Output::HTML::Layout' => {
+                    SetCookies => {
+                        SessionIDCookie => $Self->{ParamObject}->SetCookie(
+                            Key      => $Param{SessionName},
+                            Value    => '',
+                            Expires  => '-1y',
+                            Path     => $Self->{ConfigObject}->Get('ScriptAlias'),
+                            Secure   => scalar $CookieSecureAttribute,
+                            HTTPOnly => 1,
+                        ),
+                    },
+                    %Param,
                 },
-                %{$Self},
-                %Param,
             );
 
+            $Kernel::OM->ObjectsDiscard( Objects => ['Kernel::Output::HTML::Layout'] );
+            my $LayoutObject = $Kernel::OM->Get('Kernel::Output::HTML::Layout');
+
             # create AuthObject
-            my $AuthObject = Kernel::System::Auth->new( %{$Self} );
+            my $AuthObject = $Kernel::OM->Get('Kernel::System::Auth');
             if ( $AuthObject->GetOption( What => 'PreAuth' ) ) {
 
                 # automatic re-login
                 $Param{RequestedURL} = $LayoutObject->LinkEncode( $Param{RequestedURL} );
                 print $LayoutObject->Redirect(
-                    OP => "?Action=Login&RequestedURL=$Param{RequestedURL}",
+                    OP => "?Action=PreLogin&RequestedURL=$Param{RequestedURL}",
                 );
                 return;
             }
@@ -724,8 +808,9 @@ sub Run {
             # show login
             $LayoutObject->Print(
                 Output => \$LayoutObject->Login(
-                    Title   => 'Login',
-                    Message => $Self->{SessionObject}->SessionIDErrorMessage(),
+                    Title => 'Login',
+                    Message =>
+                        $LayoutObject->{LanguageObject}->Translate( $Self->{SessionObject}->SessionIDErrorMessage() ),
                     %Param,
                 ),
             );
@@ -740,11 +825,7 @@ sub Run {
         # check needed data
         if ( !$UserData{UserID} || !$UserData{UserLogin} || $UserData{UserType} ne 'User' ) {
 
-            # new layout object
-            my $LayoutObject = Kernel::Output::HTML::Layout->new(
-                %{$Self},
-                Lang => $Param{Lang},
-            );
+            my $LayoutObject = $Kernel::OM->Get('Kernel::Output::HTML::Layout');
 
             # redirect to alternate login
             if ( $Self->{ConfigObject}->Get('LoginURL') ) {
@@ -769,17 +850,13 @@ sub Run {
         my $ModuleReg = $Self->{ConfigObject}->Get('Frontend::Module')->{ $Param{Action} };
         if ( !$ModuleReg ) {
 
-            # new layout object
-            my $LayoutObject = Kernel::Output::HTML::Layout->new(
-                %{$Self},
-                Lang => $Param{Lang},
-            );
             $Self->{LogObject}->Log(
                 Priority => 'error',
                 Message =>
                     "Module Kernel::Modules::$Param{Action} not registered in Kernel/Config.pm!",
             );
-            $LayoutObject->FatalError( Comment => 'Please contact your administrator' );
+            $Kernel::OM->Get('Kernel::Output::HTML::Layout')
+                ->FatalError( Comment => 'Please contact your administrator' );
             return;
         }
 
@@ -789,18 +866,20 @@ sub Run {
             $Param{AccessRw} = 1;
         }
         else {
+            PERMISSION:
             for my $Permission (qw(GroupRo Group)) {
                 my $AccessOk = 0;
                 my $Group    = $ModuleReg->{$Permission};
                 my $Key      = "UserIs$Permission";
-                next if !$Group;
+                next PERMISSION if !$Group;
                 if ( ref $Group eq 'ARRAY' ) {
+                    INNER:
                     for ( @{$Group} ) {
-                        next if !$_;
-                        next if !$UserData{ $Key . "[$_]" };
-                        next if $UserData{ $Key . "[$_]" } ne 'Yes';
+                        next INNER if !$_;
+                        next INNER if !$UserData{ $Key . "[$_]" };
+                        next INNER if $UserData{ $Key . "[$_]" } ne 'Yes';
                         $AccessOk = 1;
-                        last;
+                        last INNER;
                     }
                 }
                 else {
@@ -819,32 +898,31 @@ sub Run {
             }
             if ( !$Param{AccessRo} && !$Param{AccessRw} || !$Param{AccessRo} && $Param{AccessRw} ) {
 
-                # new layout object
-                my $LayoutObject = Kernel::Output::HTML::Layout->new(
-                    %{$Self},
-                    Lang => $Param{Lang},
-                );
-                print $LayoutObject->NoPermission(
+                print $Kernel::OM->Get('Kernel::Output::HTML::Layout')->NoPermission(
                     Message => 'No Permission to use this frontend module!'
                 );
                 return;
             }
         }
 
-        # create new LayoutObject with new '%Param' and '%UserData'
-        my $LayoutObject = Kernel::Output::HTML::Layout->new(
-            %{$Self},
-            %Param,
-            %UserData,
-            ModuleReg => $ModuleReg,
+        # put '%Param' and '%UserData' into LayoutObject
+        $Kernel::OM->ObjectParamAdd(
+            'Kernel::Output::HTML::Layout' => {
+                %Param,
+                %UserData,
+                ModuleReg => $ModuleReg,
+            },
         );
+        $Kernel::OM->ObjectsDiscard( Objects => ['Kernel::Output::HTML::Layout'] );
 
-        # updated last request time
-        $Self->{SessionObject}->UpdateSessionID(
-            SessionID => $Param{SessionID},
-            Key       => 'UserLastRequest',
-            Value     => $Self->{TimeObject}->SystemTime(),
-        );
+        # update last request time
+        if ( !$Self->{ParamObject}->IsAJAXRequest() ) {
+            $Self->{SessionObject}->UpdateSessionID(
+                SessionID => $Param{SessionID},
+                Key       => 'UserLastRequest',
+                Value     => $Self->{TimeObject}->SystemTime(),
+            );
+        }
 
         # pre application module
         my $PreModule = $Self->{ConfigObject}->Get('PreApplicationModule');
@@ -856,10 +934,12 @@ sub Run {
             else {
                 $PreModuleList{Init} = $PreModule;
             }
+
+            MODULE:
             for my $PreModuleKey ( sort keys %PreModuleList ) {
                 my $PreModule = $PreModuleList{$PreModuleKey};
-                next if !$PreModule;
-                next if !$Self->{MainObject}->Require($PreModule);
+                next MODULE if !$PreModule;
+                next MODULE if !$Self->{MainObject}->Require($PreModule);
 
                 # debug info
                 if ( $Self->{Debug} ) {
@@ -869,6 +949,8 @@ sub Run {
                     );
                 }
 
+                my $LayoutObject = $Kernel::OM->Get('Kernel::Output::HTML::Layout');
+
                 # use module
                 my $PreModuleObject = $PreModule->new(
                     %{$Self},
@@ -898,7 +980,7 @@ sub Run {
             %{$Self},
             %Param,
             %UserData,
-            LayoutObject => $LayoutObject,
+            LayoutObject => $Kernel::OM->Get('Kernel::Output::HTML::Layout'),
             ModuleReg    => $ModuleReg,
         );
 
@@ -911,7 +993,7 @@ sub Run {
         }
 
         # ->Run $Action with $GenericObject
-        $LayoutObject->Print( Output => \$GenericObject->Run() );
+        $Kernel::OM->Get('Kernel::Output::HTML::Layout')->Print( Output => \$GenericObject->Run() );
 
         # log request time
         if ( $Self->{ConfigObject}->Get('PerformanceLog') ) {
@@ -949,12 +1031,13 @@ sub Run {
     my %Data = $Self->{SessionObject}->GetSessionIDData(
         SessionID => $Param{SessionID},
     );
-    my $LayoutObject = Kernel::Output::HTML::Layout->new(
-        %{$Self},
-        %Param,
-        %Data,
+    $Kernel::OM->ObjectParamAdd(
+        'Kernel::Output::HTML::Layout' => {
+            %Param,
+            %Data,
+        },
     );
-    $LayoutObject->FatalError( Comment => 'Please contact your administrator' );
+    $Kernel::OM->Get('Kernel::Output::HTML::Layout')->FatalError( Comment => 'Please contact your administrator' );
     return;
 }
 
