@@ -1,5 +1,6 @@
 // --
-// Copyright (C) 2001-2017 OTRS AG, http://otrs.com/
+// Core.Customer.Login.js - provides functions for the customer login
+// Copyright (C) 2001-2012 OTRS AG, http://otrs.org/
 // --
 // This software comes with ABSOLUTELY NO WARRANTY. For details, see
 // the enclosed file COPYING for license information (AGPL). If you
@@ -62,10 +63,11 @@ Core.Customer.Login = (function (TargetNS) {
             $SliderNavigationLinks = $('#Slider a');
 
         // Browser is too old
-        if (!Core.Debug.BrowserCheckCustomer()) {
+        if (!Core.Customer.SupportedBrowser) {
             $('#Login').hide();
             $('#Reset').hide();
             $('#Signup').hide();
+            $('#PreLogin').hide();
             $('#OldBrowser').show();
             return;
         }
@@ -75,6 +77,11 @@ Core.Customer.Login = (function (TargetNS) {
 
         $('#TimeOffset').val(Diff);
 
+        if ( $('#PreLogin').length ) {
+            $('#PreLogin form').submit();
+            return;
+        }
+
         $Inputs
             .focus(function () {
                 $Label = $(this).prev('label');
